/*
**  SLOPE_FOR: C-XSC classes for one- and multidimensional (interval)
**             slope arithmetic in forward mode
**
**  Copyright (C) 1999-2001 M. Braeuer, W. Hofschuster, W. Kraemer
**                          Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//////////////////////////////////////////////////////////////////////////
//
// Headerfile mit Hilfsfunktionen fuer eindim. und mehrdim. 
// Intervallsteigungsarithmetik
//
//////////////////////////////////////////////////////////////////////////

#ifndef _HELPFUNCS_H
#define _HELPFUNCS_H

//cxsc headers
#include "interval.hpp"
#include "imath.hpp"

//C++ standard headers
#include <iostream.h>
#include <stdlib.h>

//////////////////////////////////////////////////////////////////////////
//
// Definitionen der globalen Hilfsfunktionen
//
//////////////////////////////////////////////////////////////////////////

namespace cxsc {

int even(int n);

interval h_abs(const real&, const interval& );
interval h_max(const interval&, const interval&);
interval chi(const interval&, const interval&, const interval&);


int s_1_down(const interval&, const interval&, real&, interval(*f)(const interval& ));
int s_1_up(const interval&, const interval&, real&, interval(*f)(const interval& ));

int s_2_down(const interval&, const interval&, real&, interval(*f)(const interval& ));
int s_2_up(const interval&, const interval&, real&, interval(*f)(const interval& ));


int s_1_down(const interval&, const interval&, int , real&,
	     interval(*f)(const interval& , const int ));
int s_1_up(const interval&, const interval&, int, real&,
	   interval(*f)(const interval& , const int));

int s_2_down(const interval&, const interval&, int, real&,
	     interval(*f)(const interval& , const int));
int s_2_up(const interval&, const interval&, int, real&,
	   interval(*f)(const interval& , const int));

} // end namespace cxsc 

#endif
