/*
**  SLOPE_FOR: C-XSC classes for one- and multidimensional (interval)
**             slope arithmetic in forward mode
**
**  Copyright (C) 1999-2001 M. Braeuer, W. Hofschuster, W. Kraemer
**                          Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include "interval.hpp"
#include "imatrix.hpp"
#include "rmatrix.hpp"

#include "slope_ari.h" //mehrdim. Steigungsarithmetik einbinden

using namespace std;
using namespace cxsc;

/*---------------------------------------------------------------------------

Funktion f definieren, fuer die Nullstellen verifiziert werden sollen

---------------------------------------------------------------------------*/
slope f1(slope_vector x)
{
 slope res;
 res=sqr(x[1])-exp(x[2])+abs(x[2]);
 return res;
}

slope f2(slope_vector x)
{
 slope res;
 res=sqr(x[1])-sqr(x[2])-abs(x[1]);
 return res;
}

//-------------------------------------------------------------------------

int main()
{
   slope_vector x;
   slope_vector y(1,2,2);

   int booly=1;
   int counter=1;

   ivector x_ein(2); //beinhaltet geschaetzte Intervalle,
                     // wo NS liegen koennten

   x_ein[1]=_interval(-3.0, succ(-2.9));
   x_ein[2]=_interval( pred(2.4),  2.5);

   rvector c_ein(2);
   
   while(booly)
     {
       c_ein[1]=mid(x_ein[1]); //mit Mittelpkt.-en initialisieren
       c_ein[2]=mid(x_ein[2]);

       x=InitVar(x_ein, c_ein);

       y[1]=f1(x);
       y[2]=f2(x);

       //print_slope(y[1]);
       //print_slope(y[2]); 

       ivector s1(2); s1=get_fs(y[1]);
       ivector s2(2); s2=get_fs(y[2]);  
   
       //cout << s1 << " " << s2 << " " << endl;

       ivector  x_approx(2);
       ivector fx_approx(2);
       ivector xnew(2);

       rmatrix I(2,2), M(2,2), R(2,2); 

       real det;

       imatrix slopemx(2,2);

       /*-------------------------------------------------------------------
	 I, M, R berechnen / initialisieren
	 -----------------------------------------------------------------*/ 

       I[1][1]=1.0 ;   I[1][2]=0.0;
       I[2][1]=0.0 ;   I[2][2]=1.0;

       M[1][1]=mid(s1[1]);   M[1][2]=mid(s1[2]);
       M[2][1]=mid(s2[1]);   M[2][2]=mid(s2[2]);   

       //nun: R:=inv(M)

       det=M[1][1]*M[2][2]-M[1][2]*M[2][1];

       if(det==0) cout << "Matrix nicht invertierbar" << endl;

       R[1][1]=M[2][2]/det;    R[1][2]=-M[1][2]/det;
       R[2][1]=-M[2][1]/det;   R[2][2]=M[1][1]/det;
   
       slopemx[1][1]=s1[1]; slopemx[1][2]=s1[2];
       slopemx[2][1]=s2[1]; slopemx[2][2]=s2[2];

       /*-------------------------------------------------------------------
	 x_approx  und Funktionswert bestimmen
       -------------------------------------------------------------------*/ 

       x_approx[1]=get_fc(x[1]);
       x_approx[2]=get_fc(x[2]);

       //cout << "Entwicklungspunkte: " << endl << x_approx << endl;
   
       fx_approx[1]= get_fx(f1(Init_mid_Var(x_approx)));       
       fx_approx[2]= get_fx(f2(Init_mid_Var(x_approx)));

       /*-------------------------------------------------------------------
	 Nun Fixpunkt verifizieren
       -------------------------------------------------------------------*/

       xnew=x_approx-R*fx_approx+(I-R*slopemx)*(x_ein-x_approx);

      /*-----------------------------------------------------------------
	 Ausgabe
       -------------------------------------------------------------------*/ 

       cout << counter << endl;
       if ((Inf(xnew[1]) > Inf(x_ein[1])) && (Sup(xnew[1])<Sup(x_ein[1]))) 
	 cout << " erste Komp.    Ja" << endl;
       else
	 cout << " erste Komp.    Nein" << endl;
  
       if ((Inf(xnew[2]) > Inf(x_ein[2])) && (Sup(xnew[2])<Sup(x_ein[2]))) 
	 cout << " zweite Komp.   Ja" << endl;
       else
	 cout << " zweite Komp.   Nein" << endl;

       if( (diam(xnew[1])<=1e-14)
	    && 
	    (diam(xnew[2])<=1e-14) )
	 {cout << "Precision erreicht" << endl; booly=0;}

       if(counter>5)  booly=0; 

      x_ein=xnew;
      counter++;

      cout << SetPrecision(8,6) << endl;
      cout <<x_ein << endl;;

     }//while

}

