/*
**  SLOPE_FOR: C-XSC classes for one- and multidimensional (interval)
**             slope arithmetic in forward mode
**
**  Copyright (C) 1999-2001 M. Braeuer, W. Hofschuster, W. Kraemer
**                          Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include "slope_ari.h"

using namespace std;
using namespace cxsc;

slope_type f1(const slope_type& s)
{
 slope_type erg;
 erg=power(s,4)-_real(10.0)*power(s,3)+_real(35.0)*sqr(s);
 erg=erg-_real(50.0)*s+_real(24.0);
 return erg;
}

slope fn(const slope_vector& x)
{
 slope erg;
 erg=_real(4.0)*sqr(x[1])-_interval(21)/_interval(10)*power(x[1],4)+
      _interval(1.0)/_interval(3.0)*power(x[1],6)+
      x[1]*x[2]-_real(4.0)*sqr(x[2])+_real(4.0)*power(x[2],4);
 return erg;
}

slope f2(const slope_vector& x)
{
 slope erg;
 erg=_real(4.0)*ln(x[1])-_interval(21)/_interval(10)*sqr(x[1])*ln(x[1])+
      _interval(1.0)/_interval(3.0)*sqr(x[1])*sqr(x[1])*ln(x[1])+
      x[1]*x[2]-_real(4.0)*sqr(x[2])+_real(4.0)*sqr(x[2])*ln(x[2]);
 //erg=x[1]+x[2]+acot(x[2])*sin(x[1]);
 return erg;
}


int main(){

  //Beispielprogramm fuer Einsatz der vorgefertigten Prozeduren

  //eindimensionale Steigungsarithmetik

  interval arg=_interval(0.75,1.75);
  interval f_v;

  feval(f1, arg , f_v);

  cout << endl <<"Argument: " << arg << endl;
  cout <<"Funktionswert: " << f_v << endl;

  interval sl;
  fseval(f1, arg, f_v, sl);
  
  cout << endl;
  cout <<"Argument: " << arg << endl;
  cout <<"Funktionswert: " << f_v << endl;   
  cout <<"Steigung: " << sl << endl << endl;


  //mehrdimensionale Steigungsarithmetik

  ivector x_ein(2); //Intervallvektor mit Werten der Abhaengigen

  x_ein[1]=_interval(8,8.25); //Initialisiere x_ein 
  x_ein[2]=_interval(3,3.25);  

  feval(f2, x_ein, f_v);

  cout <<"Argumentvektor: " << endl << x_ein;
  cout <<"Funktionswert: " << f_v << endl;

  ivector sls(2);
  fseval(fn, x_ein, f_v, sls);
  
  cout << endl;
  cout <<"Argumentvektor: " << endl << x_ein;
  cout <<"Funktionswert: " << f_v << endl;   
  cout <<"Steigung: " << endl << sls << endl;


} 

