/*
**  SLOPE_FOR: C-XSC classes for one- and multidimensional (interval)
**             slope arithmetic in forward mode
**
**  Copyright (C) 1999-2001 M. Braeuer, W. Hofschuster, W. Kraemer
**                          Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//////////////////////////////////////////////////////////////////////////
//
// Headerfile slope.h fuer eindimensionale Intervallsteigungsarithmetik
// v2.0 mit Fallunterscheidungen
//
//////////////////////////////////////////////////////////////////////////

#ifndef _SLOPE_H
#define _SLOPE_H

#include "helpfuncs.h"

namespace cxsc {

//////////////////////////////////////////////////////////////////////////
//
//Definitionen der Klassen
//
//////////////////////////////////////////////////////////////////////////

/*-----------------------------------------------------------------------

class slope_type  zur automatischen Bestimmung der Intervallsteigung einer
                 gegebenen Funktion
                 angelehnt an PASCAL-XSC Version von Dr. D. Ratz (slopes.p)

  Elemente: 
            interval  f_x ............... 1.Intervalkomponente (Fkt.wert)
            interval  f_c ............... 2.Intervalkomponente (Fkt.wert)
	    interval  f_s ............... Steigung

            static int calc_slope_type ...falls 1 werden Steigungen  be-
	                                  rechnet; default ist 1

  Elementfunktionen:

            siehe Implementation 

Implementationsfile: slope.cpp

-----------------------------------------------------------------------*/

class slope_type{

  private:
   interval f_x; //f(X)
   interval f_c; //f(C)   
   interval f_s; //zugehoerige Steigungskomponente

   static int calc_slope_type;
 
  public:
   static void set_calc_slope_type();
   static void unset_calc_slope_type();
   static int get_calc_slope_type();

   slope_type(){;};
   slope_type(const real& );
   slope_type(const interval& );
   slope_type(const interval& , const interval&);
   slope_type(const interval& , const real&);

   friend slope_type var_slope_type(const interval&, const interval& );

   friend slope_type const_slope_type(const real& );
   friend slope_type const_slope_type(const interval& );
   friend slope_type const_slope_type(const interval&, const interval& );
   friend slope_type const_slope_type(const interval&, const real& );

   friend slope_type init_mid_var(const interval& );
   friend slope_type init_mid_const(const interval& );

   friend interval get_fx(const slope_type& );
   friend interval get_fc(const slope_type& );
   friend interval get_fs(const slope_type& );

   friend void print_slope_type(const slope_type& );

   friend slope_type operator-(const slope_type& );
  
   friend slope_type operator-(const slope_type& , const slope_type& );
   friend slope_type operator+(const slope_type& , const slope_type& );
   friend slope_type operator*(const slope_type& , const slope_type& );
   friend slope_type operator/(const slope_type& , const slope_type& );

   friend slope_type operator-(const slope_type& , const interval& );
   friend slope_type operator+(const slope_type& , const interval& );
   friend slope_type operator*(const slope_type& , const interval& );
   friend slope_type operator/(const slope_type& , const interval& );

   friend slope_type operator-(const interval& , const slope_type& );
   friend slope_type operator+(const interval& , const slope_type& );
   friend slope_type operator*(const interval& , const slope_type& );
   friend slope_type operator/(const interval& , const slope_type& );  

   friend slope_type operator-(const slope_type& , const real& );
   friend slope_type operator+(const slope_type& , const real& );
   friend slope_type operator*(const slope_type& , const real& );
   friend slope_type operator/(const slope_type& , const real& );

   friend slope_type operator-(const real& , const slope_type& );
   friend slope_type operator+(const real& , const slope_type& );
   friend slope_type operator*(const real& , const slope_type& );
   friend slope_type operator/(const real& , const slope_type& ); 

   friend slope_type sqr(const slope_type&);
   friend slope_type sqrt(const slope_type&);
   friend slope_type sqrt(const slope_type&, int);

   friend slope_type power(const slope_type&, int);
   friend slope_type pow(const slope_type&, const interval& );

   friend slope_type ln(const slope_type&); 
   friend slope_type exp(const slope_type&); 

   friend slope_type sin(const slope_type&); 
   friend slope_type cos(const slope_type&);
   friend slope_type tan(const slope_type&);
   friend slope_type cot(const slope_type&);

   friend slope_type asin(const slope_type&); 
   friend slope_type acos(const slope_type&);
   friend slope_type atan(const slope_type&);
   friend slope_type acot(const slope_type&);

   friend slope_type sinh(const slope_type&); 
   friend slope_type cosh(const slope_type&);
   friend slope_type tanh(const slope_type&);
   friend slope_type coth(const slope_type&);

   friend slope_type asinh(const slope_type&); 
   friend slope_type acosh(const slope_type&);
   friend slope_type atanh(const slope_type&);
   friend slope_type acoth(const slope_type&);

   friend slope_type abs(const slope_type&);
   friend slope_type max(const slope_type&, const slope_type&);
   friend slope_type min(const slope_type&, const slope_type&);
   friend slope_type chi(const slope_type&, const slope_type&, const slope_type&);
};

void feval(slope_type (*f)( const slope_type& ), const interval& ,  interval& );
void fseval(slope_type (*f)( const slope_type& ), const interval&, 
	    interval& ,  interval& );

} // namespace cxsc

#endif
