/*
**  SLOPE_FOR: C-XSC classes for one- and multidimensional (interval)
**             slope arithmetic in forward mode
**
**  Copyright (C) 1999-2001 M. Braeuer, W. Hofschuster, W. Kraemer
**                          Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include "slope_ari.h"

using namespace std;
using namespace cxsc; 

slope f1(const slope_vector& x)
{
 slope erg;
 erg=_interval(100)*sqr(x[2]-sqr(x[1]))+sqr(x[1]-_interval(1.0));
 return erg;
}

slope f2(const slope_vector& x)
{
 slope erg;
 erg=_interval(4.0)*sqr(x[1])+x[1]*x[2]+_interval(4.0)*sqr(x[2])-x[1];
 return erg;
}

slope f3(const slope_vector& x)
{
 slope erg;
 erg=exp(x[1]*x[2])-x[1];
 return erg;
}

slope f4(const slope_vector& x)
{
 slope erg;
 erg=x[1]-exp(x[2])+sqr(sin(x[2]));
 return erg;
}

slope f5(const slope_vector& x)
{
 slope erg;
 erg=abs(x[1]-exp(x[2])+sqr(sin(x[2])));
 return erg;
}


int main()
{

  /*----------------------------------------------------------------------

    Berechnung von Steigungen verschiedener Funktionen

  ----------------------------------------------------------------------*/

  cout << Scientific << SetPrecision(8,6) ;

 ivector x_in(2), c_in(2); 
 ivector slopes(2);

 interval fx_val;
 interval fc_val;

 x_in[1]=_interval(4.0,4.25);
 x_in[2]=_interval(4.0,4.25);

 c_in[1]=_interval(4.125, 4.125);
 c_in[2]=_interval(4.125, 4.125);

 slope_vector x;
 slope y;
 
 /*-----------------------------------------------------------------------
 Auswertung der Funktion f1
 -----------------------------------------------------------------------*/

 x=InitVar(x_in, c_in);
 y=f1(x); 

 fx_val=get_fx(y);
 fc_val=get_fc(y);
 slopes=get_fs(y);

 cout << "Funktion 1" << endl;
 cout << "Funktionswert x: " << fx_val << endl;
 cout << "Funktionswert c: " << fc_val << endl;

 cout << "Slopes :" << endl;
 cout << slopes << endl;
 
 cout << "zentrische Form:" << fc_val+slopes*(x_in-c_in)<< endl; 
 cout << endl;

 /*-----------------------------------------------------------------------
 Auswertung der Funktion f2
 -----------------------------------------------------------------------*/

 x=InitVar(x_in, c_in);
 y=f2(x); 

 fx_val=get_fx(y);
 fc_val=get_fc(y);
 slopes=get_fs(y);

 cout << "Funktion 2" << endl;
 cout << "Funktionswert x: " << fx_val << endl;
 cout << "Funktionswert c: " << fc_val << endl;

 cout << "Slopes :" << endl;
 cout << slopes << endl;
 
 cout << "zentrische Form:" << fc_val+slopes*(x_in-c_in)<< endl; 
 cout << endl;

 /*-----------------------------------------------------------------------
 Auswertung der Funktion f3
 -----------------------------------------------------------------------*/

 x_in[1]=_interval(-1.0,1.0);
 x_in[2]=_interval(0.0,2.0);

 c_in[1]=_interval(0.0);
 c_in[2]=_interval(1.0);

 x=InitVar(x_in, c_in);
 y=f3(x); 

 fx_val=get_fx(y);
 fc_val=get_fc(y);
 slopes=get_fs(y);

 cout << "Funktion 3" << endl;
 cout << "Funktionswert x: " << fx_val << endl;
 cout << "Funktionswert c: " << fc_val << endl;

 cout << "Slopes :" << endl;
 cout << slopes << endl;
 
 cout << "zentrische Form:" << fc_val+slopes*(x_in-c_in)<< endl; 
 cout << endl;

 /*-----------------------------------------------------------------------
 Auswertung der Funktion f4
 -----------------------------------------------------------------------*/

 x_in[1]=_interval(-0.25,0.25);
 x_in[2]=_interval(1.0,1.25);

 c_in[1]=_interval(0.0);
 c_in[2]=_interval(1.125);

 x=InitVar(x_in, c_in);
 y=f4(x); 

 fx_val=get_fx(y);
 fc_val=get_fc(y);
 slopes=get_fs(y);

 cout << "Funktion 4" << endl;
 cout << "Funktionswert x: " << fx_val << endl;
 cout << "Funktionswert c: " << fc_val << endl;

 cout << "Slopes :" << endl;
 cout << slopes << endl;
 
 cout << "zentrische Form:" << fc_val+slopes*(x_in-c_in)<< endl; 
 cout << endl; 

 /*-----------------------------------------------------------------------
 Auswertung der Funktion f5
 -----------------------------------------------------------------------*/

 x=InitVar(x_in, c_in);
 y=f5(x); 

 fx_val=get_fx(y);
 fc_val=get_fc(y);
 slopes=get_fs(y);


 cout << "Funktion 5" << endl;
 cout << "Funktionswert x: " << fx_val << endl;
 cout << "Funktionswert c: " << fc_val << endl;

 cout << "Slopes :" << endl;
 cout << slopes << endl;
 
 cout << "zentrische Form:" << fc_val+slopes*(x_in-c_in)<< endl; 
 cout << endl; 
} 


/*

VORWAERTS:

Funktion 1
Funktionswert x: [1.381525E+004,1.978596E+004]
Funktionswert c: [1.662658E+004,1.662659E+004]
Slopes :
[2.002663E+004,2.257962E+004]
[-2.695313E+003,-2.464062E+003]

zentrische Form:[1.346722E+004,1.978596E+004]

------------------------------------------------

Funktion 2
Funktionswert x: [1.397500E+002,1.585625E+002]
Funktionswert c: [1.490156E+002,1.490157E+002]
Slopes :
[ 35.500000, 36.750000]
[ 36.625000, 37.625000]

zentrische Form:[1.397187E+002,1.583125E+002]

------------------------------------------------

Funktion 3
Funktionswert x: [ -0.864665,  8.389057]
Funktionswert c: [  1.000000,  1.000000]
Slopes :
[ -1.000000,  5.389057]
[ -0.000000,  0.000000]

zentrische Form:[ -4.389057,  6.389057]

------------------------------------------------

Funktion 4
Funktionswert x: [ -3.032270, -1.567710]
Funktionswert c: [ -2.266131, -2.266130]
Slopes :
[  1.000000,  1.000000]
[ -2.731170, -1.895244]

zentrische Form:[ -2.857527, -1.674733]

------------------------------------------------

Funktion 5
Funktionswert x: [  1.567710,  3.032270]
Funktionswert c: [  2.266130,  2.266131]
Slopes :
[ -1.000000, -1.000000]
[  1.895244,  2.731170]

zentrische Form:[  1.674733,  2.857527]


------------------------------------------------

RUECKWAERTS:

Funktion 1
Funktionswert x: [1.381525E+004,1.978596E+004]
Funktionswert c: [1.662658E+004,1.662659E+004]
Slopes :
[2.002663E+004,2.257962E+004]
[-2.695313E+003,-2.464062E+003]

zentrische Form:[1.346722E+004,1.978596E+004]

-----------------------------------------------------

Funktion 2
Funktionswert x: [1.397500E+002,1.585625E+002]
Funktionswert c: [1.490156E+002,1.490157E+002]
Slopes :
[ 35.500000, 36.750000]
[ 36.625000, 37.625000]

zentrische Form:[1.397187E+002,1.583125E+002]

-----------------------------------------------------

Funktion 3
Funktionswert x: [ -0.864665,  8.389057]
Funktionswert c: [  1.000000,  1.000000]
Slopes :
[ -1.000000,  5.389057]
[  0.000000,  0.000000]

zentrische Form:[ -4.389057,  6.389057]

-----------------------------------------------------

Funktion 4
Funktionswert x: [ -3.032270, -1.567710]
Funktionswert c: [ -2.266131, -2.266130]
Slopes :
[  1.000000,  1.000000]
[ -2.731170, -1.895244]

zentrische Form:[ -2.857527, -1.674733]

-----------------------------------------------------

Funktion 5
Funktionswert x: [  1.567710,  3.032270]
Funktionswert c: [  2.266130,  2.266131]
Slopes :
[ -1.000000, -1.000000]
[  1.895244,  2.731170]

zentrische Form:[  1.674733,  2.857527]


--------------------------------------------------------

KOMPONENTENWEISE BERECHNUNG

Funktion 1
Funktionswert x: [1.381525E+004,1.978596E+004]
Funktionswert c: [1.662658E+004,1.662659E+004]
Slopes :
[2.012819E+004,2.247493E+004]
[-2.800000E+003,-2.362500E+003]

zentrische Form:[1.346722E+004,1.978596E+004]

------------------------------------------------------

Funktion 2
Funktionswert x: [1.397500E+002,1.583125E+002]
Funktionswert c: [1.490156E+002,1.490157E+002]
Slopes :
[ 35.625000, 36.625000]
[ 36.500000, 37.750000]

zentrische Form:[1.397187E+002,1.583125E+002]

------------------------------------------------------

Funktion 3
Funktionswert x: [ -0.864665,  6.389057]
Funktionswert c: [  1.000000,  1.000000]
Slopes :
[ -0.367880,  0.718282]
[ -4.670775,  4.670775]

zentrische Form:[ -4.389057,  6.389057]

------------------------------------------------------

Funktion 4
Funktionswert x: [ -2.857527, -1.674733]
Funktionswert c: [ -2.266131, -2.266130]
Slopes :
[  1.000000,  1.000000]
[ -2.731170, -1.895244]

zentrische Form:[ -2.857527, -1.674733]

------------------------------------------------------

Funktion 5
Funktionswert x: [  1.674733,  2.857527]
Funktionswert c: [  2.266130,  2.266131]
Slopes :
[ -1.000000, -1.000000]
[  1.895244,  2.731170]

zentrische Form:[  1.674733,  2.857527]

--------------------------------------------------------------------

Scientific << SetPrecision(20,18) ;

Vorwaerts 
Funktion 1
Funktionswert x: [1.381525000000000000E+004,1.978595312500000000E+004]
Funktionswert c: [1.662658691406250000E+004,1.662658691406250000E+004]
Slopes :
[2.002663281250000000E+004,2.257961718750000000E+004]
[-2.695312500000000000E+003,-2.464062500000000000E+003]

zentrische Form:[1.346722070312500000E+004,1.978595312500000000E+004]

Funktion 2
Funktionswert x: [1.397500000000000000E+002,1.585625000000000000E+002]
Funktionswert c: [1.490156250000000000E+002,1.490156250000000000E+002]
Slopes :
[3.550000000000000000E+001,3.675000000000000000E+001]
[3.662500000000000000E+001,3.762500000000000000E+001]

zentrische Form:[1.397187500000000000E+002,1.583125000000000000E+002]

Funktion 3
Funktionswert x: [-8.646647167633874088E-001,8.389056098930650407E+000]
Funktionswert c: [1.000000000000000000E+000,1.000000000000000000E+000]
Slopes :
[-1.000000000000000000E+000,5.389056098930650407E+000]
[-0.000000000000000000E+000,0.000000000000000000E+000]

zentrische Form:[-4.389056098930650407E+000,6.389056098930650407E+000]

Funktion 4
Funktionswert x: [-3.032269539188270358E+000,-1.567710020685577854E+000]
Funktionswert c: [-2.266130037556662203E+000,-2.266130037556661314E+000]
Slopes :
[1.000000000000000000E+000,1.000000000000000000E+000]
[-2.731169100249762228E+000,-1.895244323998821922E+000]

zentrische Form:[-2.857526175087882870E+000,-1.674733900025440869E+000]

Funktion 5
Funktionswert x: [1.567710020685577854E+000,3.032269539188270358E+000]
Funktionswert c: [2.266130037556661314E+000,2.266130037556662203E+000]
Slopes :
[-1.000000000000000000E+000,-1.000000000000000000E+000]
[1.895244323998821922E+000,2.731169100249762228E+000]

zentrische Form:[1.674733900025440869E+000,2.857526175087882870E+000]

Rueckwaerts:
Funktion 1
Funktionswert x: [1.381525000000000000E+004,1.978595312500000000E+004]
Funktionswert c: [1.662658691406250000E+004,1.662658691406250000E+004]
Slopes :
[2.002663281250000000E+004,2.257961718750000000E+004]
[-2.695312500000000000E+003,-2.464062500000000000E+003]

zentrische Form:[1.346722070312500000E+004,1.978595312500000000E+004]

Funktion 2
Funktionswert x: [1.397500000000000000E+002,1.585625000000000000E+002]
Funktionswert c: [1.490156250000000000E+002,1.490156250000000000E+002]
Slopes :
[3.550000000000000000E+001,3.675000000000000000E+001]
[3.662500000000000000E+001,3.762500000000000000E+001]

zentrische Form:[1.397187500000000000E+002,1.583125000000000000E+002]

Funktion 3
Funktionswert x: [-8.646647167633874088E-001,8.389056098930650407E+000]
Funktionswert c: [1.000000000000000000E+000,1.000000000000000000E+000]
Slopes :
[-1.000000000000000000E+000,5.389056098930650407E+000]
[0.000000000000000000E+000,0.000000000000000000E+000]

zentrische Form:[-4.389056098930650407E+000,6.389056098930650407E+000]

Funktion 4
Funktionswert x: [-3.032269539188270358E+000,-1.567710020685577854E+000]
Funktionswert c: [-2.266130037556662203E+000,-2.266130037556661314E+000]
Slopes :
[1.000000000000000000E+000,1.000000000000000000E+000]
[-2.731169100249762228E+000,-1.895244323998821922E+000]

zentrische Form:[-2.857526175087882870E+000,-1.674733900025440869E+000]

Funktion 5
Funktionswert x: [1.567710020685577854E+000,3.032269539188270358E+000]
Funktionswert c: [2.266130037556661314E+000,2.266130037556662203E+000]
Slopes :
[-1.000000000000000000E+000,-1.000000000000000000E+000]
[1.895244323998821922E+000,2.731169100249762228E+000]

zentrische Form:[1.674733900025440869E+000,2.857526175087882870E+000]


*/
